/*
    This file is part of Bemused.

    Bemused is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    Bemused is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with Bemused; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	Copyright (C) 2002 Ashley Montanaro.
*/

#include "stdafx.h"
#include "BemusedServerDlg.h"
#include "WinampHandler.h"
#include "Winamp3Handler.h"
#include "MediaPlayerHandler.h"
#include "PowerPointHandler.h"
#include "PowerPoint2003Handler.h"
#include "PocketSlidesHandler.h"
#include "PocketPlayerHandler.h"
#include "SetupWizard.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

#define ID_TASKBARICON 100
#define WM_ICONNOTIFY (WM_USER + 101)

const TCHAR* KIniFileName = _T("\\BemusedServer.ini");
const TCHAR* KIniFileSection = _T("BemusedServer");

const TCHAR* KDefaultExtensions[] = { _T("cpt"), _T("mp3;mp2;wav;wma;ogg"), _T(""), _T("ppt"), _T("") };

#ifdef UNDER_CE
extern CBemusedServerApp theApp;

DWORD GetPrivateProfileString(LPCTSTR lpAppName, LPCTSTR lpKeyName, LPCTSTR lpDefault, LPTSTR lpReturnedString, DWORD nSize, LPCTSTR lpFileName)
{
	CString ret = theApp.GetProfileString(lpAppName, lpKeyName, lpDefault);
	_tcsncpy(lpReturnedString, ret, nSize/sizeof(TCHAR));
	return lstrlen(lpReturnedString);
}

DWORD GetCurrentDirectory(DWORD nBufferLength, LPTSTR lpBuffer)
{
	GetModuleFileName(NULL, lpBuffer, nBufferLength);
	if (_tcsrchr(lpBuffer, '\\')) *_tcsrchr(lpBuffer, '\\') = 0;
	return lstrlen(lpBuffer);
}

DWORD WritePrivateProfileString(LPCTSTR lpAppName, LPCTSTR lpKeyName, LPCTSTR lpString, LPCTSTR lpFileName)
{
	return (DWORD)theApp.WriteProfileString(lpAppName, lpKeyName, lpString);
}

int GetWindowTextA(HWND hWnd, LPSTR buf, int count)
{
	CString text;
	LPTSTR lpsz = text.GetBuffer(count+2);
	::GetWindowText(hWnd, lpsz, count);
	wcstombs(buf, text, count+1);
	text.ReleaseBuffer();
	return strlen(buf);
}

#endif

#define DEBUG_LOG(x)	fwrite(x, strlen(x), 1, file); \
						fflush(file);

/////////////////////////////////////////////////////////////////////////////
// CAboutDlg dialog used for App About

class CAboutDlg : public CDialog
{
public:
	CAboutDlg();

// Dialog Data
	//{{AFX_DATA(CAboutDlg)
	enum { IDD = IDD_ABOUTBOX };
	//}}AFX_DATA

	// ClassWizard generated virtual function overrides
	//{{AFX_VIRTUAL(CAboutDlg)
	protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	//}}AFX_VIRTUAL

// Implementation
protected:
	//{{AFX_MSG(CAboutDlg)
	//}}AFX_MSG
	DECLARE_MESSAGE_MAP()
};

CAboutDlg::CAboutDlg() : CDialog(CAboutDlg::IDD)
{
	//{{AFX_DATA_INIT(CAboutDlg)
	//}}AFX_DATA_INIT
}

void CAboutDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CAboutDlg)
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CAboutDlg, CDialog)
	//{{AFX_MSG_MAP(CAboutDlg)
		// No message handlers
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CDirTreeNode class

CDirTreeNode::CDirTreeNode()
	{
	iFirstChild = NULL;
	iNextSibling = NULL;
	iParent = NULL;
	iFileName = NULL;
	iNodeType = iFileType = 0x0;
	iWrittenToBuffer = FALSE;
	}

CDirTreeNode::~CDirTreeNode()
	{
	delete iFirstChild;
	delete iNextSibling;
	delete iFileName;
	}

/////////////////////////////////////////////////////////////////////////////
// CBemusedServerDlg dialog

CBemusedServerDlg::CBemusedServerDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CBemusedServerDlg::IDD, pParent)
	{
	//{{AFX_DATA_INIT(CBemusedServerDlg)
		// NOTE: the ClassWizard will add member initialization here
	//}}AFX_DATA_INIT
	// Note that LoadIcon does not require a subsequent DestroyIcon in Win32
	m_hIcon = AfxGetApp()->LoadIcon(IDR_MAINFRAME);

	iDirTreeRoot = iDirTreeCurrentNode = NULL;
	iBluetoothHandle = INVALID_HANDLE_VALUE;
	iBluetoothThread = NULL;
	iSocketHandle = -1;
	iSocketThread = NULL;
	iErrorCount = 0;
	}

CBemusedServerDlg::~CBemusedServerDlg()
	{
	for (int i = 0; i < NUM_HANDLERS; i++)
		delete iAppHandlers[i];

/*	DEBUG_LOG("Terminating thread\r\n");

	if (iBluetoothThread)
		TerminateThread(iBluetoothThread->m_hThread, 0);
*/
	delete iBluetoothThread;
	delete iSocketThread;
	if (iDirTreeRoot) delete iDirTreeRoot;
	}

BOOL CBemusedServerDlg::OnInitDialog()
	{
	CDialog::OnInitDialog();

	// Add "About..." menu item to system menu.

	// IDM_ABOUTBOX must be in the system command range.
	ASSERT((IDM_ABOUTBOX & 0xFFF0) == IDM_ABOUTBOX);
	ASSERT(IDM_ABOUTBOX < 0xF000);
#ifndef UNDER_CE
	CMenu* pSysMenu = GetSystemMenu(FALSE);
	if (pSysMenu != NULL)
	{
		CString strAboutMenu;
		strAboutMenu.LoadString(IDS_ABOUTBOX);
		if (!strAboutMenu.IsEmpty())
		{
			pSysMenu->AppendMenu(MF_SEPARATOR);
			pSysMenu->AppendMenu(MF_STRING, IDM_ABOUTBOX, strAboutMenu);
		}
	}
#endif
	// Set the icon for this dialog.  The framework does this automatically
	//  when the application's main window is not a dialog
	SetIcon(m_hIcon, TRUE);			// Set big icon
	SetIcon(m_hIcon, FALSE);		// Set small icon

	CoInitializeEx(NULL, COINIT_MULTITHREADED);

	Log("Welcome to Bemused v%d.%d%s\r\n", MAJOR_VERSION, MINOR_VERSION, PATCH_VERSION);
	Log("Copyright Ashley Montanaro 2003\r\n");

	// Add ourselves to the system tray.
	iNotifyIconData.cbSize = sizeof(NOTIFYICONDATA);
	iNotifyIconData.hWnd = m_hWnd;
	iNotifyIconData.uID = ID_TASKBARICON;
	iNotifyIconData.uFlags = NIF_ICON | NIF_MESSAGE | NIF_TIP;
	iNotifyIconData.uCallbackMessage = WM_ICONNOTIFY;
	iNotifyIconData.hIcon = (HICON) LoadImage(AfxGetInstanceHandle(), MAKEINTRESOURCE(IDI_TRAYICON),
		IMAGE_ICON, 16, 16, 0);
	_tcscpy(iNotifyIconData.szTip, _T("Bemused Server"));

	Shell_NotifyIcon(NIM_ADD, &iNotifyIconData);
	
	GetCurrentDirectory(sizeof(iStartDirectory), iStartDirectory);

	if (theApp.GetProfileInt(KIniFileSection, _T("FirstRun"), 1))
		{
		CSetupWizard setupWizard;
		setupWizard.SetParentDialog(this);
		if (!setupWizard.DoModal())
			{
			delete setupWizard;
			exit(0);
			}
		else
			{
			delete setupWizard;
			}
		}
	theApp.WriteProfileInt(KIniFileSection, _T("FirstRun"), 0);

	ReadIniFile();

	if (_tcscmp(iCommPortName, _T("")))
		BluetoothConnect(iCommPortName);

	if (iBluetoothHandle != INVALID_HANDLE_VALUE)
		iBluetoothThread = AfxBeginThread(BluetoothThreadFunction, this);

	if (iListenOnSocket)
		ListenOnSocket();

#ifdef UNDER_CE
	iAppHandlers[0] = new CPocketSlidesHandler;
	iAppHandlers[1] = new CPocketPlayerHandler;
#else
	iAppHandlers[0] = new CWinampHandler;
	iAppHandlers[1] = new CWinamp3Handler;
	iAppHandlers[2] = new CMediaPlayerHandler;
	iAppHandlers[3] = new CPowerPointHandler;
	iAppHandlers[4] = new CPowerPoint2003Handler;
#endif
	iAppHandler = iAppHandlers[iDefaultAppIndex];
	iCurrentAppIndex = iDefaultAppIndex;

	SetTimer(TIMER_ID, 1000, NULL);

	return TRUE;  // return TRUE unless you set the focus to a control
	}

void CBemusedServerDlg::StopListeningOnSocket()
	{
	if (iSocketThread)
		{
		TerminateThread(iSocketThread->m_hThread, 0);
		delete iSocketThread;
		iSocketThread = NULL;
		}

	if (iSocketHandle != -1)
		closesocket(iSocketHandle);

	iSocketHandle = -1;
	}

void CBemusedServerDlg::ListenOnSocket()
	{
	StopListeningOnSocket();

	// Start Winsock.
	WSADATA wsaData;
	WSAStartup(MAKEWORD(1,1), &wsaData);

	iSocketHandle = socket(AF_INET,		// Address family
					   SOCK_STREAM,  // Socket type
					   IPPROTO_TCP);// Protocol

	//
	// Fill in the address structure
	//
	SOCKADDR_IN sockAddr;

	sockAddr.sin_family = AF_INET;
	sockAddr.sin_addr.s_addr = INADDR_ANY; // Let WinSock assign address
	sockAddr.sin_port = htons(7654);	   // Always use this port for now

	//
	// bind the name to the socket
	//
	int nRet;

	nRet = bind(iSocketHandle,				// Socket descriptor
				(LPSOCKADDR)&sockAddr,  // Address to bind to
				sizeof(struct sockaddr)	// Size of address
				);

	if (nRet == SOCKET_ERROR)
		{
		Log("Bind failed with %d", nRet);
		closesocket(iSocketHandle);
		iSocketHandle = -1;
		return;
		}

	nRet = listen(iSocketHandle,					// Bound socket
				  1);					// Number of connection request queue

	if (nRet == SOCKET_ERROR)
		{
		Log("Listen failed with %d", nRet);
		closesocket(iSocketHandle);
		iSocketHandle = -1;
		return;
		}

	if (iSocketHandle != -1)
		iSocketThread = AfxBeginThread(SocketThreadFunction, this);
	}

LRESULT CBemusedServerDlg::WindowProc(UINT message, WPARAM wParam, LPARAM lParam)
	{
	switch (message)
		{
	case WM_ICONNOTIFY:
		{
		if (lParam == WM_LBUTTONUP)
			{
			if (!iHidden)
				{
				iHidden = TRUE;
				ShowWindow(SW_HIDE);
				}
			else
				{
				iHidden = FALSE;
				ShowWindow(SW_SHOW);
#ifdef UNDER_CE
				// TODO
				SetForegroundWindow();
#else
				WINDOWPLACEMENT placement;
				placement.length = sizeof(placement);
				placement.flags = 0;
				placement.showCmd = SW_RESTORE;
				RECT pos;
				pos.top = 100;
				pos.left = 100;
				pos.right = 550;
				pos.bottom = 645;
				placement.rcNormalPosition = pos;
				SetWindowPlacement(&placement);
#endif
				}
			return 0;
			}
		}
		break;

	case WM_TIMER:
		iAppHandler->OneSecondTick();
		return 0;
		break;

	default:
		break;
		}

	return CWnd::WindowProc(message, wParam, lParam);
	}

void CBemusedServerDlg::DoDataExchange(CDataExchange* pDX)
	{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CBemusedServerDlg)
		// NOTE: the ClassWizard will add DDX and DDV calls here
	//}}AFX_DATA_MAP
	}

BEGIN_MESSAGE_MAP(CBemusedServerDlg, CDialog)
	//{{AFX_MSG_MAP(CBemusedServerDlg)
	ON_WM_SYSCOMMAND()
	ON_WM_PAINT()
	ON_WM_QUERYDRAGICON()
	ON_BN_CLICKED(IDC_RECONNECT, OnReconnect)
	ON_WM_CLOSE()
	ON_BN_CLICKED(IDC_EXIT, OnExit)
	ON_WM_SIZE()
	ON_BN_CLICKED(IDC_BROWSEPATH, OnBrowsePath)
	ON_BN_CLICKED(IDC_BROWSEWINAMP, OnBrowseWinamp)
	ON_BN_CLICKED(IDC_ADDPATH, OnAddpath)
	ON_BN_CLICKED(IDC_CLEARPATH, OnClearpath)
	ON_CBN_SELCHANGE(IDC_APPCOMBO, OnSelchangeAppcombo)
	ON_BN_CLICKED(IDC_FINDCOMPORT, OnFindcomport)
#ifndef UNDER_CE
	ON_WM_ENDSESSION()
#endif
	ON_BN_CLICKED(IDC_STOPWHENSWITCHING, OnStopwhenswitching)
	ON_BN_CLICKED(IDC_LISTENONSOCKET, OnListenonsocket)
	ON_WM_ACTIVATE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

void CBemusedServerDlg::WriteIniFile()
	{
	TCHAR path[MAX_PATH + 20];

	_tcscpy(path, iStartDirectory);
	_tcscat(path, KIniFileName);

	TCHAR buffer[256];

	WritePrivateProfileString(KIniFileSection, _T("CommPort"), iCommPortName, path);

	SendDlgItemMessage(IDC_EDITPATH, WM_GETTEXT, sizeof(iRootPath)/sizeof(TCHAR), (LPARAM)iRootPath);
	WritePrivateProfileString(KIniFileSection, _T("Path"), iRootPath, path);

	// Get whether to start applications automatically.
	int startAutomatically = SendDlgItemMessage(IDC_STARTWINAMP, BM_GETCHECK, 0, 0);
	swprintf(buffer, _T("%d"), startAutomatically);
	WritePrivateProfileString(KIniFileSection, _T("StartWinampAutomatically"), buffer, path);

	// Write which application is default.
	iDefaultAppIndex = SendDlgItemMessage(IDC_APPCOMBO, CB_GETCURSEL, 0, 0);
	swprintf(buffer, _T("%d"), iDefaultAppIndex);
	WritePrivateProfileString(KIniFileSection, _T("DefaultApplication"), buffer, path);

	for (int i = 0; i < NUM_HANDLERS; i++)
		{
		swprintf(buffer, _T("PathToApplication%d"), i);

		if (i == iDefaultAppIndex)
			{
			TCHAR currentPath[MAX_PATH];
			SendDlgItemMessage(IDC_WINAMPPATH, WM_GETTEXT, sizeof(currentPath)/sizeof(TCHAR), (LPARAM) currentPath);
			WritePrivateProfileString(KIniFileSection, buffer, currentPath, path);

			swprintf(buffer, _T("ExtensionsHandled%d"), i);
			SendDlgItemMessage(IDC_FILETYPESEDIT, WM_GETTEXT, sizeof(currentPath)/sizeof(TCHAR), (LPARAM) currentPath);
			WritePrivateProfileString(KIniFileSection, buffer, currentPath, path);
			}
		else
			{
			WritePrivateProfileString(KIniFileSection, buffer, iAppPaths[i], path);
			
			swprintf(buffer, _T("ExtensionsHandled%d"), i);
			WritePrivateProfileString(KIniFileSection, buffer, iExtensionsHandled[i], path);
			}
		}

	// Get whether to start this application minimised or not.
	int startMinimised = SendDlgItemMessage(IDC_STARTMINIMISED, BM_GETCHECK, 0, 0);
	swprintf(buffer, _T("%d"), startMinimised);
	WritePrivateProfileString(KIniFileSection, _T("StartMinimised"), buffer, path);

	// Get whether to stop in previous app when switching apps
	int stopWhenSwitching = SendDlgItemMessage(IDC_STOPWHENSWITCHING, BM_GETCHECK, 0, 0);
	swprintf(buffer, _T("%d"), stopWhenSwitching);
	WritePrivateProfileString(KIniFileSection, _T("StopWhenSwitching"), buffer, path);

	// Get whether to listen on an Internet socket
	int listenOnSocket = SendDlgItemMessage(IDC_LISTENONSOCKET, BM_GETCHECK, 0, 0);
	swprintf(buffer, _T("%d"), listenOnSocket);
	WritePrivateProfileString(KIniFileSection, _T("ListenOnSocket"), buffer, path);
	}

void CBemusedServerDlg::ReadIniFile()
	{
	TCHAR path[MAX_PATH + 20];
	
	_tcscpy(path, iStartDirectory);
	_tcscat(path, KIniFileName);

	TCHAR buffer[256];

	// Get which comm port to use.
	GetPrivateProfileString(KIniFileSection, _T("CommPort"), _T(""), iCommPortName, sizeof(iCommPortName), path);
	SendDlgItemMessage(IDC_COMPORT, WM_SETTEXT, 0, (LPARAM)iCommPortName);

	// Get the path to the user's music.
	GetPrivateProfileString(KIniFileSection, _T("Path"), _T(""), iRootPath, sizeof(iRootPath), path);
	if ((_tcslen(iRootPath) > 0) && (iRootPath[_tcslen(iRootPath)-1] == '\\'))
		iRootPath[_tcslen(iRootPath)-1] = '\0';

	SendDlgItemMessage(IDC_EDITPATH, WM_SETTEXT, 0, (LPARAM)iRootPath);

	// Get whether to start Winamp automatically.
	int startWinampAutomatically = 0;
	GetPrivateProfileString(KIniFileSection, _T("StartWinampAutomatically"), _T("0"), buffer, sizeof(buffer), path);
	swscanf(buffer, _T("%d"), &startWinampAutomatically);

	SendDlgItemMessage(IDC_STARTWINAMP, BM_SETCHECK, startWinampAutomatically, 0);

	// Get whether to start minimised.
	GetPrivateProfileString(KIniFileSection, _T("StartMinimised"), _T("0"), buffer, sizeof(buffer), path);
	swscanf(buffer, _T("%d"), &iHidden);
	SendDlgItemMessage(IDC_STARTMINIMISED, BM_SETCHECK, iHidden, 0);

	// Get whether to stop when switching.
	int stopWhenSwitching = 0;
	GetPrivateProfileString(KIniFileSection, _T("StopWhenSwitching"), _T("0"), buffer, sizeof(buffer), path);
	swscanf(buffer, _T("%d"), &stopWhenSwitching);
	SendDlgItemMessage(IDC_STOPWHENSWITCHING, BM_SETCHECK, stopWhenSwitching, 0);

	// Get whether to listen on an Internet socket.
	GetPrivateProfileString(KIniFileSection, _T("ListenOnSocket"), _T("0"), buffer, sizeof(buffer), path);
	swscanf(buffer, _T("%d"), &iListenOnSocket);
	SendDlgItemMessage(IDC_LISTENONSOCKET, BM_SETCHECK, iListenOnSocket, 0);

	// Get which application is the default.
	GetPrivateProfileString(KIniFileSection, _T("DefaultApplication"), _T("0"), buffer, sizeof(buffer), path);
	swscanf(buffer, _T("%d"), &iDefaultAppIndex);

	SendDlgItemMessage(IDC_APPCOMBO, CB_SETCURSEL, iDefaultAppIndex, 0);

	for (int i = 0; i < NUM_HANDLERS; i++)
		{
		swprintf(buffer, _T("PathToApplication%d"), i);
		GetPrivateProfileString(KIniFileSection, buffer, _T(""), iAppPaths[i],
			sizeof(iAppPaths[i]), path);

		if (i == iDefaultAppIndex)
			SendDlgItemMessage(IDC_WINAMPPATH, WM_SETTEXT, 0, (LPARAM) iAppPaths[i]);

		swprintf(buffer, _T("ExtensionsHandled%d"), i);
		GetPrivateProfileString(KIniFileSection, buffer, KDefaultExtensions[i], iExtensionsHandled[i],
			sizeof(iExtensionsHandled[i]), path);

		if (i == iDefaultAppIndex)
			SendDlgItemMessage(IDC_FILETYPESEDIT, WM_SETTEXT, 0, (LPARAM) iExtensionsHandled[i]);
		}
	}


UINT CBemusedServerDlg::BluetoothThreadFunction(LPVOID pParam)
	{
	CBemusedServerDlg* self = (CBemusedServerDlg*) pParam;
	self->HandleBluetooth();
	return 0;
	}

UINT CBemusedServerDlg::SocketThreadFunction(LPVOID pParam)
	{
	CBemusedServerDlg* self = (CBemusedServerDlg*) pParam;
	self->HandleSocket();
	return 0;
	}

void CBemusedServerDlg::HandleBluetooth()
	{
	iExitNow = FALSE;
	do {
		ReadCommand(iBluetoothHandle);

		if (iErrorCount >= 10)
			{
			Log("Too many errors; exiting\r\n");
			break;
			}

		} while (!iExitNow);

	CloseHandle(iBluetoothHandle);
	}

void CBemusedServerDlg::HandleSocket()
	{
	iExitNow = FALSE;

	SOCKET readSocket;

	do {
		iClientClosed = FALSE;

		readSocket = accept(iSocketHandle,			// Listening socket
					  NULL,					// Optional client address
					  NULL);

		Log("Client connected on Internet socket\r\n");

		do {
			ReadCommand(NULL, readSocket);

			if (iErrorCount >= 10)
				{
				Log("Too many errors; exiting\r\n");
				
				closesocket(readSocket);
				closesocket(iSocketHandle);
				return;
				}

			} while (!iClientClosed);

		closesocket(readSocket);

		} while (!iExitNow);

	closesocket(iSocketHandle);
	}

void CBemusedServerDlg::ReadCommand(HANDLE aHandle, SOCKET aSocket)
	{
	char buffer[5];
	unsigned long bytes_read = 0;

	memset(buffer, 0, sizeof(buffer));

	BOOL readOk = TRUE;

	if (aHandle)
		{
			COMMTIMEOUTS cto;
			GetCommTimeouts(aHandle, &cto);
			cto.ReadIntervalTimeout = 200;
			cto.ReadTotalTimeoutConstant = 200;
			cto.ReadTotalTimeoutMultiplier = 100;
			SetCommTimeouts(aHandle, &cto);
			SetCommMask(aHandle, EV_RXCHAR);
			DWORD dwMask = 0;
			if (TRUE) {//WaitCommEvent(aHandle, &dwMask, NULL) && dwMask & EV_RXCHAR) {
				readOk = ReadFile(aHandle, buffer, 4, &bytes_read, NULL);
				if (!readOk && GetLastError() == ERROR_INVALID_HANDLE) {
					iErrorCount++;
				} else if (!readOk && bytes_read == 0) {
					int err = GetLastError();
					readOk = TRUE;
					Sleep(100);
				}
			} else {
				buffer[0] = 0;
				readOk = TRUE;
				Sleep(50);
			}
		}
	else
		{
		int nRet;
		char* bufPtr = buffer;

		do {
			nRet = recv(aSocket,			// Bound socket
					bufPtr,					// Receive buffer
					4 - (bufPtr - buffer),	// Size of buffer in bytes
					0);						// Flags

			if (nRet == 0)
				{
				Log("Client has closed connection\r\n");
				iClientClosed = TRUE;
				return;
				}
			else if (nRet < 0)
				{
				readOk = FALSE;
				break;
				}

			bufPtr += nRet;

			} while ((bufPtr - buffer) < 4);
		}

	if (!readOk)
		{
		Log("Couldn't read command: %d\r\n", GetLastError());
	//	iErrorCount++;
		}
	else if (buffer[0])
		{
#ifdef DEBUG
		Log(buffer);
#endif
		if (strcmp(buffer, "PLAY") && strcmp(buffer, "CHCK") && strcmp(buffer, "VERS"))
			CheckAppRunning();

		if (!strcmp(buffer, "CHCK"))
			SendAck(aSocket);
		else if (!strcmp(buffer, "LIST"))
			WriteListToPhone(aSocket);
		else if (!strcmp(buffer, "DLST"))
			WriteDirectoryListToPhone(aSocket);
		else if (!strcmp(buffer, "INFO"))
			WriteInfoToPhone(aSocket);
		else if (!strcmp(buffer, "INF2"))
			WriteInf2ToPhone(aSocket);
		else if (!strcmp(buffer, "PLAY"))
			PlayFile(FALSE, aSocket);
		else if (!strcmp(buffer, "LADD"))
			PlayFile(TRUE, aSocket);
		else if (!strcmp(buffer, "FINF"))
			GetFileInfo(aSocket);
		else if (!strcmp(buffer, "DOWN"))
			DownloadFile(aSocket);
		else if (!strcmp(buffer, "VOLM"))
			SetVolume(aSocket);
		else if (!strcmp(buffer, "GVOL"))
			GetVolume(aSocket);
		else if (!strcmp(buffer, "STOP"))
			Stop(EImmediate);
		else if (!strcmp(buffer, "FADE"))
			Stop(EFadeOut);
		else if (!strcmp(buffer, "STEN"))
			Stop(EAtEndOfSong);
		else if (!strcmp(buffer, "PAUS"))
			Pause();
		else if (!strcmp(buffer, "STRT"))
			Play();
		else if (!strcmp(buffer, "PREV"))
			PreviousTrack();
		else if (!strcmp(buffer, "NEXT"))
			NextTrack();
		else if (!strcmp(buffer, "FFWD"))
			FastForward();
		else if (!strcmp(buffer, "RWND"))
			Rewind();
		else if (!strcmp(buffer, "SHFL"))
			Shuffle(aSocket);
		else if (!strcmp(buffer, "REPT"))
			Repeat(aSocket);
		else if (!strcmp(buffer, "PLST"))
			WritePlaylist(aSocket);
		else if (!strcmp(buffer, "SLCT"))
			SelectInPlaylist(aSocket);
		else if (!strcmp(buffer, "RMAL"))
			RemoveAllFromPlaylist();
		else if (!strcmp(buffer, "DINF"))
			GetDetailedInfo(aSocket);
		else if (!strcmp(buffer, "SHUT"))
			ShutDown();
		else if (!strcmp(buffer, "FULL"))
			FullScreen();
		else if (!strcmp(buffer, "SEEK"))
			SetSeek(aSocket);
		else if (!strcmp(buffer, "PLEN"))
			WritePlaylistLength(aSocket);
		else if (!strcmp(buffer, "VERS"))
			WriteVersion(aSocket);
		else if (!strcmp(buffer, "EXIT"))
			exit(0);
		else
			Log("Read unknown command \"%s\"\r\n", buffer);
		}
	}

void CBemusedServerDlg::ReadDir(TCHAR* aDir, CDirTreeNode* aNode, BOOL aRecursive)
	{
	if (!aDir)
		{
		Log("Couldn't open null directory!\r\n");
		return;
		}

	TCHAR fileNameBuf[1024];
	swprintf(fileNameBuf, _T("%s\\*.*"), aDir);

	WIN32_FIND_DATA findData;
	int foundCount = 0;
	HANDLE findHandle = FindFirstFile(fileNameBuf, &findData);

	if (findHandle == INVALID_HANDLE_VALUE)
		{
		// empty dir, should never happen?
		///Log("Couldn't open \"%S\"\r\n", aDir);
		return;
		}

	CDirTreeNode* currentNode = aNode;

	do {
		if (!_tcscmp(findData.cFileName, _T(".")) || !_tcscmp(findData.cFileName, _T("..")))
			continue;

		if (foundCount == 0)
			{
			aNode->iFirstChild = new CDirTreeNode;
			aNode->iFirstChild->iParent = currentNode;
			currentNode = aNode->iFirstChild;
			currentNode->iNodeType = 0x1; // child
			}
		else
			{
			currentNode->iNextSibling = new CDirTreeNode;
			currentNode->iNextSibling->iParent = currentNode->iParent;
			currentNode = currentNode->iNextSibling;
			currentNode->iNodeType = 0x3; // sibling
			}

		foundCount++;

		swprintf(fileNameBuf, _T("%s\\%s"), aDir, findData.cFileName);
		currentNode->iFileName = _tcsdup(findData.cFileName);

		if (findData.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
			{
			if (aRecursive)
				{
				currentNode->iFileType = 0x1;
				ReadDir(fileNameBuf, currentNode, TRUE);
				}
			else
				{
				currentNode->iFileType = 0x3;
				}
			}
		else
			{
			currentNode->iFileType = 0x2;
			}

		} while (FindNextFile(findHandle, &findData));

	if (foundCount > 0)
		{
		// if the last node was a child, make it an only child
		if (currentNode->iNodeType == 0x1)
			currentNode->iNodeType = 0x2;
		// if the last node was a sibling, make it the last sibling
		if (currentNode->iNodeType == 0x3)
			currentNode->iNodeType = 0x4;
		}

	FindClose(findHandle);
	}

// Gets a handle to Winamp, starting it if necessary. Returns whether we retrieved the
// handle successfully.
BOOL CBemusedServerDlg::CheckAppRunning()
	{
	// Get whether to start Winamp automatically.
	int startAutomatically = SendDlgItemMessage(IDC_STARTWINAMP, BM_GETCHECK, 0, 0);

	return iAppHandler->CheckRunning(iAppPaths[iCurrentAppIndex], startAutomatically);
	}

void CBemusedServerDlg::WriteInfoToPhone(SOCKET aSocket)
	{
	unsigned char isPlaying;
	int songLength;
	int currentTime;
	BOOL shuffle;
	BOOL repeat;
	char songNameGuess[256];
	
	memset(songNameGuess, 0, sizeof(songNameGuess));

	iAppHandler->GetInfo(isPlaying, songLength, currentTime, shuffle, repeat, songNameGuess,
		sizeof(songNameGuess));

//	Log("Playing: %d; length=%d, current=%d, Song name guess: %s\r\n",
//		isPlaying, songLength, currentTime, songNameGuess);

	char buffer[1024];

	strcpy(buffer, "INFOACK");
	buffer[7] = isPlaying;

	buffer[8] = songLength >> 24;
	buffer[9] = (songLength >> 16) & 0xFF;
	buffer[10] = (songLength >> 8) & 0xFF;
	buffer[11] = songLength & 0xFF;

	buffer[12] = currentTime >> 24;
	buffer[13] = (currentTime >> 16) & 0xFF;
	buffer[14] = (currentTime >> 8) & 0xFF;
	buffer[15] = currentTime & 0xFF;

	buffer[16] = shuffle;
	buffer[17] = repeat;

	strcpy(&buffer[18], songNameGuess);

	unsigned long bytesWritten;

	if (aSocket)
		bytesWritten = send(aSocket, buffer, 18 + strlen(songNameGuess), 0);
	else
		WriteFile(iBluetoothHandle, buffer, 18 + strlen(songNameGuess), &bytesWritten, NULL);

	if (bytesWritten < 18)
		Log("Only wrote %d bytes of INFOACK\r\n", bytesWritten);
	}

void CBemusedServerDlg::WriteInf2ToPhone(SOCKET aSocket)
	{
	unsigned char isPlaying;
	int songLength;
	int currentTime;
	BOOL shuffle;
	BOOL repeat;
	char songNameGuess[256];

	iAppHandler->GetInfo(isPlaying, songLength, currentTime, shuffle, repeat, songNameGuess,
		sizeof(songNameGuess), FALSE);

	char buffer[1024];

	memset(buffer, 0, sizeof(buffer));

	strcpy(buffer, "INF2ACK");
	buffer[7] = isPlaying;

	buffer[8] = songLength >> 24;
	buffer[9] = (songLength >> 16) & 0xFF;
	buffer[10] = (songLength >> 8) & 0xFF;
	buffer[11] = songLength & 0xFF;

	buffer[12] = currentTime >> 24;
	buffer[13] = (currentTime >> 16) & 0xFF;
	buffer[14] = (currentTime >> 8) & 0xFF;
	buffer[15] = currentTime & 0xFF;

	buffer[16] = shuffle;
	buffer[17] = repeat;

	strcpy(&buffer[18], songNameGuess);

	unsigned long bytesWritten;

	if (aSocket)
		bytesWritten = send(aSocket, buffer, 19 + strlen(songNameGuess), 0);
	else
		WriteFile(iBluetoothHandle, buffer, 19 + strlen(songNameGuess), &bytesWritten, NULL);

	if (bytesWritten < 19)
		Log("Only wrote %d bytes of INF2ACK\r\n", bytesWritten);
	}

void CBemusedServerDlg::WriteVersion(SOCKET aSocket)
	{
	char buffer[9];

	memset(buffer, 0, sizeof(buffer));

	strcpy(buffer, "VERSACK");
	buffer[7] = MAJOR_VERSION;
	buffer[8] = MINOR_VERSION;

	unsigned long bytesWritten;

	if (aSocket)
		bytesWritten = send(aSocket, buffer, 9, 0);
	else
		WriteFile(iBluetoothHandle, buffer, 9, &bytesWritten, NULL);

	Log("Wrote version to phone\r\n");
	}

void CBemusedServerDlg::PlayFile(BOOL aAddToPlaylist, SOCKET aSocket)
	{
	unsigned long bytes = 0;

	unsigned char buffer[1024];
	int filenameLength = 0;
	memset(buffer, 0, sizeof(buffer));

	if (!ReadFromPhone(aSocket, buffer, 2))
		{
		Log("Couldn't read filename length: %d\r\n", GetLastError());
		return;
		}

	filenameLength = (buffer[0] << 8) + buffer[1];
	buffer[0] = '\0';
	buffer[1] = '\0';

	if (filenameLength >= sizeof(buffer))
		{
		Log("Filename length (%d) too long", filenameLength);
		return;
		}

	if (!ReadFromPhone(aSocket, buffer, filenameLength))
		{
		Log("Couldn't read filename of length %d: %d\r\n", filenameLength, GetLastError());
		return;
		}

	CAppHandler* handler = FindHandlerForFile(CString(buffer));

	if (handler != iAppHandler)
		{
		int stopWhenSwitching = SendDlgItemMessage(IDC_STOPWHENSWITCHING, BM_GETCHECK, 0, 0);

		if (stopWhenSwitching)
			Stop(EImmediate);
		}

	iAppHandler = handler;

	CheckAppRunning();

	if (aAddToPlaylist)
		{
		Log("Adding \"%s\" to playlist\r\n", buffer);
		iAppHandler->AddSongToPlaylist((char*) buffer);
		}
	else
		{
		Log("Playing \"%s\"\r\n", buffer);
		iAppHandler->PlaySong((char*) buffer);
		}
	}

void CBemusedServerDlg::GetFileInfo(SOCKET aSocket)
	{
	unsigned long bytes = 0;

	unsigned char buffer[1024];
	int filenameLength = 0;
	memset(buffer, 0, sizeof(buffer));

	if (!ReadFromPhone(aSocket, buffer, 2))
		{
		Log("Couldn't read filename length: %d\r\n", GetLastError());
		return;
		}

	filenameLength = (buffer[0] << 8) + buffer[1];
	buffer[0] = '\0';
	buffer[1] = '\0';

	if (filenameLength >= sizeof(buffer))
		{
		Log("Filename length (%d) too long", filenameLength);
		return;
		}

	if (!ReadFromPhone(aSocket, buffer, filenameLength))
		{
		Log("Couldn't read filename: %d\r\n", GetLastError());
		return;
		}

	Log("Returning info about \"%s\": ", buffer);

	FILE* file = fopen((char*) buffer, "rb");

	if (file)
		{
		fseek(file, 0, SEEK_END);
		long fileLength = ftell(file);
		fseek(file, 0, SEEK_SET);

		Log("length is %d\r\n", fileLength);

		char ackBuf[11];
		strcpy(ackBuf, "FINFACK");
		ackBuf[7] = (fileLength >> 24) & 0xFF;
		ackBuf[8] = (fileLength >> 16) & 0xFF;
		ackBuf[9] = (fileLength >> 8) & 0xFF;
		ackBuf[10] = fileLength & 0xFF;

		if (aSocket)
			bytes = send(aSocket, ackBuf, 11, 0);
		else
			WriteFile(iBluetoothHandle, ackBuf, 11, &bytes, NULL);
		}
	else
		{
		Log("couldn't open file\r\n");
		}

	fclose(file);
	}

void CBemusedServerDlg::WritePlaylist(SOCKET aSocket)
	{
	unsigned long bytesWritten = 0;
	int playlistLength;
	int playlistPos;
	char* playlist;
	char* playlistPtr;

	Log("Writing playlist to phone\r\n");

	iAppHandler->GetPlaylist(playlist, playlistLength, playlistPos);
	playlistPtr = playlist;

	unsigned char buffer[10];
	strcpy((char*) buffer, "PLSTACK");
	buffer[7] = (playlistPos >> 8) & 0xFF;
	buffer[8] = playlistPos & 0xFF;

	if (aSocket)
		bytesWritten = send(aSocket, (char*)buffer, 9, 0);
	else
		WriteFile(iBluetoothHandle, buffer, 9, &bytesWritten, NULL);

	bytesWritten = 0;

	if (playlist == NULL)
		{
		Log("Couldn't get playlist! Playlist retrieval is only supported for Winamp 2.\r\n");

		buffer[9] = '\0';

		if (aSocket)
			bytesWritten = send(aSocket, (char*)buffer, 10, 0);
		else
			WriteFile(iBluetoothHandle, buffer, 10, &bytesWritten, NULL);
		return;
		}

	do {

		if (aSocket)
			bytesWritten = send(aSocket, playlistPtr, playlistLength, 0);
		else
			WriteFile(iBluetoothHandle, playlistPtr, playlistLength, &bytesWritten, NULL);

		if (bytesWritten < (unsigned long) playlistLength)
			{
//			Log("Only wrote part of playlist: retrying\r\n");
			playlistPtr += bytesWritten;
			playlistLength -= bytesWritten;
			bytesWritten = 0;
			}
		} while (bytesWritten < (unsigned long) playlistLength);

	delete playlist;

	Log("Finished writing playlist\r\n");
	}

void CBemusedServerDlg::DownloadFile(SOCKET aSocket)
	{
	unsigned long bytes = 0;

	unsigned char buffer[1024];
	unsigned char* bufferPtr;
	int filenameLength = 0;
	memset(buffer, 0, sizeof(buffer));

	if (!ReadFromPhone(aSocket, buffer, 2))
		{
		Log("Couldn't read filename length: %d\r\n", GetLastError());
		return;
		}

	filenameLength = (buffer[0] << 8) + buffer[1];
	buffer[0] = '\0';
	buffer[1] = '\0';

	if (filenameLength >= sizeof(buffer))
		{
		Log("Filename length (%d) too long", filenameLength);
		return;
		}

	if (!ReadFromPhone(aSocket, buffer, filenameLength))
		{
		Log("Couldn't read filename: %d\r\n", GetLastError());
		return;
		}

	Log("Uploading \"%s\"... ", buffer);

	FILE* file = fopen((char*) buffer, "rb");
	unsigned long bytesRead;

	if (file)
		{
		char ackBuf[8];
		strcpy(ackBuf, "DOWNACK");

		if (aSocket)
			bytes = send(aSocket, ackBuf, 7, 0);
		else
			WriteFile(iBluetoothHandle, ackBuf, 7, &bytes, NULL);

		do {
			bytesRead = fread(buffer, 1, sizeof(buffer), file);
			bufferPtr = buffer;
			bytes = 0;

			if (bytesRead > 0)
				{
				do {
					bytesRead -= bytes;
					bufferPtr += bytes;

					if (aSocket)
						bytes = send(aSocket, (char*)bufferPtr, bytesRead, 0);
					else
						WriteFile(iBluetoothHandle, bufferPtr, bytesRead, &bytes, NULL);

					} while ((bytes > 0) && (bytes < bytesRead));
//				if (bytes < bytesRead)
//					Log("wrote %d bytes of %d read from file\r\n", bytes, bytesRead);
				}
			} while (bytesRead > 0);

		Log("finished\r\n");
		}
	else
		{
		Log("couldn't open file\r\n");
		}

	fclose(file);
	}

void CBemusedServerDlg::SetVolume(SOCKET aSocket)
	{
	unsigned long bytesRead = 0;
	unsigned char volume = 0;

	if (!ReadFromPhone(aSocket, &volume, 1))
		{
		Log("Couldn't read volume: %d\r\n", GetLastError());
		return;
		}

	// set the volume
	iAppHandler->SetVolume(volume);

	Log("Set volume to %d%%\r\n", (volume * 100) / 255);
	}

void CBemusedServerDlg::GetVolume(SOCKET aSocket)
	{
	int volume = iAppHandler->GetVolume();
	char buffer[9];
	unsigned long bytesWritten;

	if (volume >= 0)
		{
		strcpy(buffer, "GVOLACK");
		buffer[7] = (char) volume;

		if (aSocket)
			bytesWritten = send(aSocket, buffer, 8, 0);
		else
			WriteFile(iBluetoothHandle, buffer, 8, &bytesWritten, NULL);

		Log("Wrote volume (%2.2f%%) to phone\r\n", (volume * 100.0) / 255.0);
		}
	else
		{
		strcpy(buffer, "GVOLNAK");

		if (aSocket)
			bytesWritten = send(aSocket, buffer, 7, 0);
		else
			WriteFile(iBluetoothHandle, buffer, 7, &bytesWritten, NULL);

		Log("Couldn't get volume (ret=%d)\r\n", volume);
		}
	}

void CBemusedServerDlg::Play()
	{
	iAppHandler->Play();
	Log("Playing\r\n");
	}

void CBemusedServerDlg::Stop(TStopMode aStopMode)
	{
	iAppHandler->Stop(aStopMode);
	Log("Stopping\r\n");
	}

void CBemusedServerDlg::Pause()
	{
	iAppHandler->Pause();
	Log("Pause\r\n");
	}

void CBemusedServerDlg::NextTrack()
	{
	iAppHandler->NextTrack();
	Log("Next track\r\n");
	}

void CBemusedServerDlg::PreviousTrack()
	{
	iAppHandler->PreviousTrack();
	Log("Previous track\r\n");
	}

void CBemusedServerDlg::FastForward()
	{
	iAppHandler->FastForward();
	Log("Fast forward\r\n");
	}

void CBemusedServerDlg::Rewind()
	{
	iAppHandler->Rewind();
	Log("Rewind\r\n");
	}

void CBemusedServerDlg::FullScreen()
	{
	iAppHandler->FullScreen();
	Log("Full screen\r\n");
	}

void CBemusedServerDlg::Shuffle(SOCKET aSocket)
	{
	unsigned long bytesRead = 0;
	unsigned char shuffleOn = 0;

	if (!ReadFromPhone(aSocket, &shuffleOn, 1))
		{
		Log("Couldn't read shuffle: %d\r\n", GetLastError());
		return;
		}

	iAppHandler->SetShuffle(shuffleOn);
	Log("Changing shuffle to %s\r\n", shuffleOn ? "on" : "off");
	}

void CBemusedServerDlg::Repeat(SOCKET aSocket)
	{
	unsigned long bytesRead = 0;
	unsigned char repeatOn = 0;

	if (!ReadFromPhone(aSocket, &repeatOn, 1))
		{
		Log("Couldn't read repeat: %d\r\n", GetLastError());
		return;
		}

	iAppHandler->SetRepeat(repeatOn);
	Log("Changing repeat to %s\r\n", repeatOn ? "on" : "off");
	}

void CBemusedServerDlg::SendAck(SOCKET aSocket)
	{
	char ackBuf = 'Y';
	unsigned long bytes;

	if (aSocket)
		bytes = send(aSocket, (char*)&ackBuf, 1, 0);
	else
		WriteFile(iBluetoothHandle, &ackBuf, 1, &bytes, NULL);

	Log("Sent ACK to phone\r\n");
	}

void CBemusedServerDlg::SelectInPlaylist(SOCKET aSocket)
	{
	unsigned long bytesRead = 0;
	unsigned char indexBytes[2];

	if (!ReadFromPhone(aSocket, indexBytes, 2))
		{
		Log("Couldn't read playlist index: %d\r\n", GetLastError());
		return;
		}

	iAppHandler->SelectInPlaylist((indexBytes[0] << 8) + indexBytes[1]);

	Log("Selected song %d in playlist\r\n", (indexBytes[0] << 8) + indexBytes[1]);
	}

void CBemusedServerDlg::RemoveAllFromPlaylist()
	{
	iAppHandler->RemoveAllFromPlaylist();
	Log("Removed all songs from playlist\r\n");
	}

void CBemusedServerDlg::GetDetailedInfo(SOCKET aSocket)
	{
	int bitRate;
	int sampleRate;
	int channels;

	iAppHandler->GetDetailedInfo(bitRate, sampleRate, channels);

//	Log("Bit rate: %d, sample rate: %d, channels: %d\r\n", bitRate, sampleRate, channels);

	char infoBuf[256];
	unsigned long bytes;

	strcpy(infoBuf, "DINFACK");

	CopyIntIntoBuffer(bitRate, &infoBuf[7]);
	CopyIntIntoBuffer(sampleRate, &infoBuf[11]);
	CopyIntIntoBuffer(channels, &infoBuf[15]);

	if (aSocket)
		bytes = send(aSocket, infoBuf, 19, 0);
	else
		WriteFile(iBluetoothHandle, &infoBuf, 19, &bytes, NULL);

	Log("Sent detailed track info to phone\r\n");
	}

BOOL CBemusedServerDlg::ReadFromPhone(SOCKET aSocket, unsigned char* aBuffer, int aLength)
	{
	BOOL success = TRUE;
	int bytes;

	if (aSocket)
		{
		bytes = recv(aSocket, (char*)aBuffer, aLength, 0);
		if (bytes <= 0)
			success = FALSE;
		}
	else
		{
		if (!ReadFile(iBluetoothHandle, aBuffer, aLength, (unsigned long*) &bytes, NULL))
			success = FALSE;
		}

	return success;
	}

void CBemusedServerDlg::CopyIntIntoBuffer(int aValue, char* aBuffer)
	{
	aBuffer[0] = (aValue >> 24) & 0xFF;
	aBuffer[1] = (aValue >> 16) & 0xFF;
	aBuffer[2] = (aValue >> 8) & 0xFF;
	aBuffer[3] = aValue & 0xFF;
	}

BOOL CBemusedServerDlg::SetPrivilege(HANDLE hToken, LPCTSTR lpszPrivilege, BOOL bEnablePrivilege)
	{
#ifndef UNDER_CE
	TOKEN_PRIVILEGES tp;
	LUID luid;

	if (!LookupPrivilegeValue( 
			NULL,            // lookup privilege on local system
			lpszPrivilege,   // privilege to lookup 
			&luid))
		{
		Log("LookupPrivilegeValue error: %u\n", GetLastError()); 
		return FALSE; 
		}

	tp.PrivilegeCount = 1;
	tp.Privileges[0].Luid = luid;
	if (bEnablePrivilege)
		tp.Privileges[0].Attributes = SE_PRIVILEGE_ENABLED;
	else
		tp.Privileges[0].Attributes = 0;

	// Enable the privilege or disable all privileges.

	AdjustTokenPrivileges(
		   hToken, 
		   FALSE, 
		   &tp, 
		   sizeof(TOKEN_PRIVILEGES), 
		   (PTOKEN_PRIVILEGES) NULL, 
		   (PDWORD) NULL); 
 
	// Call GetLastError to determine whether the function succeeded.

	if (GetLastError() != ERROR_SUCCESS)
		{ 
		Log("AdjustTokenPrivileges failed: %u\n", GetLastError()); 
		return FALSE; 
		} 
#endif
	return TRUE;
	}
 
void CBemusedServerDlg::ShutDown()
	{
#ifndef UNDER_CE
	Log("Shutting down Windows\r\n");

	// We only need to get special privileges to shut down under Windows NT (& 2000, etc.),
	// so we check the OS version here.
	if (GetVersion() < 0x80000000)
		{
		HANDLE tokenHandle;
		OpenProcessToken(GetCurrentProcess(), TOKEN_WRITE, &tokenHandle);
		SetPrivilege(tokenHandle, SE_SHUTDOWN_NAME, TRUE);
		}

	ExitWindowsEx(EWX_POWEROFF, 0);
#endif
	}

void CBemusedServerDlg::ResetWrittenFlag(CDirTreeNode* aNode)
	{
	aNode->iWrittenToBuffer = FALSE;

	if (aNode->iFirstChild)
		ResetWrittenFlag(aNode->iFirstChild);
	if (aNode->iNextSibling)
		ResetWrittenFlag(aNode->iNextSibling);
	}

void CBemusedServerDlg::WriteListToPhone(SOCKET aSocket)
	{
	unsigned long bytesWritten = 0;
	unsigned char buffer[1024];
	unsigned char* bufferPtr;

	// Get a list of the files we can browse.
	TCHAR pathBuffer[256];
	SendDlgItemMessage(IDC_EDITPATH, WM_GETTEXT, sizeof(pathBuffer)/sizeof(TCHAR), (LPARAM)pathBuffer);
	if ((_tcslen(pathBuffer) > 0) && (pathBuffer[_tcslen(pathBuffer)-1] == '\\'))
		pathBuffer[_tcslen(pathBuffer)-1] = '\0';

	GenerateFileList(pathBuffer, TRUE);

	if (aSocket)
		bytesWritten = send(aSocket, "LISTACK", 7, 0);
	else
		WriteFile(iBluetoothHandle, "LISTACK", 7, &bytesWritten, NULL);

	if (bytesWritten < 7)
		{
		Log("Couldn't write list to phone (error %d)\r\n", GetLastError());
		return;
		}

	int sizeLeft;
	int nextPos;
	BOOL allDone;

	int totalBytesWritten = 0;

	// If we don't have any files, just return end-of-list.
	if (!iDirTreeRoot)
		{
		buffer[0] = 0xFF;
		buffer[1] = 0x00;
		WriteFile(iBluetoothHandle, buffer, 2, &bytesWritten, NULL);
		return;
		}

	iDirTreeCurrentNode = iDirTreeRoot;
	ResetWrittenFlag(iDirTreeRoot);

	do {
		sizeLeft = sizeof(buffer) - 2;

		bufferPtr = buffer;
		memset(buffer, 0, sizeof(buffer));
		allDone = WriteNodeToBuffer(iDirTreeRoot, bufferPtr, sizeLeft);

		totalBytesWritten += (sizeof(buffer) - sizeLeft);
		nextPos = sizeof(buffer) - 2 - sizeLeft;

		if (allDone)
			{
			// Write "last entry" token.
			buffer[nextPos++] = 0xFF;
			buffer[nextPos++] = 0x00;
			}

		bufferPtr = buffer;
		bytesWritten = 0;
		do {

			if (aSocket)
				bytesWritten = send(aSocket, (char*)bufferPtr, nextPos, 0);
			else
				WriteFile(iBluetoothHandle, bufferPtr, nextPos, &bytesWritten, NULL);

			if (bytesWritten < (unsigned long) nextPos)
				{
//				Log("Only wrote part of list: retrying\r\n");
				bufferPtr += bytesWritten;
				nextPos -= bytesWritten;
				bytesWritten = 0;
				}
			} while (bytesWritten < (unsigned long) nextPos);

		} while (!allDone);

	Log("Wrote song list to phone (%d bytes)\r\n", totalBytesWritten);
	}

void CBemusedServerDlg::WriteDirectoryListToPhone(SOCKET aSocket)
	{
	unsigned long bytesWritten = 0;
	unsigned char buffer[1024];
	unsigned char* bufferPtr;

	int filenameLength = 0;
	memset(buffer, 0, sizeof(buffer));

	if (!ReadFromPhone(aSocket, buffer, 2))
		{
		Log("Couldn't read directory name length: %d\r\n", GetLastError());
		return;
		}

//	Log("Read %d bytes of directory name length: 0x%X 0x%X\r\n", bytes, buffer[0], buffer[1]);

	filenameLength = (buffer[0] << 8) + buffer[1];
	buffer[0] = '\0';
	buffer[1] = '\0';

	if (filenameLength >= sizeof(buffer))
		{
		Log("Filename length (%d) too long", filenameLength);
		return;
		}

	if (!ReadFromPhone(aSocket, buffer, filenameLength))
		{
		Log("Couldn't read directory name: %d\r\n", GetLastError());
		return;
		}

	if (!strcmp((char*)buffer, ""))
		{
		// Get a list of the files we can browse.
		//SendMessage(IDC_EDITPATH, WM_GETTEXT, sizeof(buffer), (LPARAM)buffer);
		GetWindowTextA((GetDlgItem(IDC_EDITPATH))->GetSafeHwnd(), (char*)buffer, sizeof(buffer));
		if ((strlen((char*)buffer) > 0) && (buffer[strlen((char*)buffer)-1] == '\\'))
			buffer[strlen((char*)buffer)-1] = '\0';
		}

	Log("Listing \"%s\": ", buffer);

	GenerateFileList(CString(buffer), FALSE);

	// Note that we return a standard LISTACK to simplify client-side handling.

	if (aSocket)
		bytesWritten = send(aSocket, "LISTACK", 7, 0);
	else
		WriteFile(iBluetoothHandle, "LISTACK", 7, &bytesWritten, NULL);

	if (bytesWritten < 7)
		{
		Log("Couldn't write list to phone (error %d)\r\n", GetLastError());
		return;
		}

	int sizeLeft;
	int nextPos;
	BOOL allDone;

	int totalBytesWritten = 0;

	// If we don't have any files, just return end-of-list.
	if (!iDirTreeRoot)
		{
		buffer[0] = 0xFF;
		buffer[1] = 0x00;

		if (aSocket)
			bytesWritten = send(aSocket, (char*)buffer, 2, 0);
		else
			WriteFile(iBluetoothHandle, buffer, 2, &bytesWritten, NULL);

		Log("no files to list\r\n");
		return;
		}

	iDirTreeCurrentNode = iDirTreeRoot;
	ResetWrittenFlag(iDirTreeRoot);

	do {
		sizeLeft = sizeof(buffer) - 2;

		bufferPtr = buffer;
		memset(buffer, 0, sizeof(buffer));
		allDone = WriteNodeToBuffer(iDirTreeRoot, bufferPtr, sizeLeft);

		totalBytesWritten += (sizeof(buffer) - sizeLeft);
		nextPos = sizeof(buffer) - 2 - sizeLeft;

		if (allDone)
			{
			// Write "last entry" token.
			buffer[nextPos++] = 0xFF;
			buffer[nextPos++] = 0x00;
			}

		bufferPtr = buffer;
		bytesWritten = 0;
		do {

			if (aSocket)
				bytesWritten = send(aSocket, (char*)bufferPtr, nextPos, 0);
			else
				WriteFile(iBluetoothHandle, bufferPtr, nextPos, &bytesWritten, NULL);

			if (bytesWritten < (unsigned long) nextPos)
				{
//				Log("Only wrote part of list: retrying\r\n");
				bufferPtr += bytesWritten;
				nextPos -= bytesWritten;
				bytesWritten = 0;
				}
			} while (bytesWritten < (unsigned long) nextPos);

		} while (!allDone);

	Log("wrote list to phone (%d bytes)\r\n", totalBytesWritten);
	}

BOOL CBemusedServerDlg::WriteNodeToBuffer(CDirTreeNode* aNode, unsigned char*& aBufferPtr, int& aSizeRemaining)
	{
	int lengthNeeded = lstrlen(aNode->iFileName) + 2;
	if (aSizeRemaining < lengthNeeded)
		return FALSE;

	unsigned char type = (aNode->iNodeType << 4) + aNode->iFileType;

	if (!aNode->iWrittenToBuffer)
		{
		aBufferPtr[0] = type;
		//memcpy(&aBufferPtr[1], aNode->iFileName, lengthNeeded - 1);
		wcstombs((char*)&aBufferPtr[1], aNode->iFileName, lengthNeeded - 1);

		aBufferPtr += lengthNeeded;
		aSizeRemaining -= lengthNeeded;

		aNode->iWrittenToBuffer = TRUE;
		}

	int ret = TRUE;

	if (ret && aNode->iFirstChild)
		ret = WriteNodeToBuffer(aNode->iFirstChild, aBufferPtr, aSizeRemaining);

	if (ret && aNode->iNextSibling)
		ret = WriteNodeToBuffer(aNode->iNextSibling, aBufferPtr, aSizeRemaining);

	return ret;
	}

void CBemusedServerDlg::GenerateFileList(const TCHAR* aRoot, BOOL aRecursive)
	{
	if (iDirTreeRoot) delete iDirTreeRoot;
	iDirTreeRoot = NULL;

	if (!_tcscmp(aRoot, _T("")))
		return;

	iDirTreeRoot = new CDirTreeNode;
	iDirTreeRoot->iFileType = 0x1; // dir
	iDirTreeRoot->iNodeType = 0x0; // root
	iDirTreeRoot->iFirstChild = iDirTreeRoot->iNextSibling = NULL;
	iDirTreeCurrentNode = iDirTreeRoot;

	// Different handling here based on whether we've only got one directory as our root,
	// or multiple directories.
	if (_tcschr(aRoot, ';'))
		{
		iDirTreeRoot->iFileName = _tcsdup(_T(""));

		TCHAR rootPath[1024];
		const TCHAR* rootPtr = aRoot;
		CDirTreeNode* newNode;
		CDirTreeNode* lastNewNode;
		int pathCount = 0;
		BOOL onLastPath = FALSE;

		while (1)
			{
			memset(rootPath, 0, sizeof(rootPath));

			if (!onLastPath)
				{
				_tcsncpy(rootPath, rootPtr, _tcschr(rootPtr, ';') - rootPtr);
				rootPtr = _tcschr(rootPtr, ';') + 1;
				}
			else
				{
				_tcscpy(rootPath, rootPtr);
				}

			// Remove a trailing backslash if there is one.
			if ((lstrlen(rootPath) > 0) && (rootPath[lstrlen(rootPath)-1] == '\\'))
				rootPath[lstrlen(rootPath)-1] = '\0';

			// Check that we've got a valid path.
			if (!_tcscmp(rootPath, _T("")))
				{
				if (!_tcschr(rootPtr, ';'))
					onLastPath = TRUE;
				continue;
				}

			pathCount++;

			newNode = new CDirTreeNode;
			newNode->iParent = iDirTreeRoot;
			newNode->iFileName = _tcsdup(rootPath);

			if (aRecursive)
				newNode->iFileType = 0x1; // dir
			else
				newNode->iFileType = 0x3; // unexpanded dir

			if (pathCount == 1)
				{
				iDirTreeRoot->iFirstChild = newNode;
				newNode->iNodeType = 0x1; // child
				}
			else
				{
				lastNewNode->iNextSibling = newNode;
				newNode->iNodeType = 0x3; // sibling
				}

			iDirTreeCurrentNode = newNode;

			if (aRecursive)
				ReadDir(newNode->iFileName, newNode, aRecursive);

			lastNewNode = newNode;

			if (onLastPath)
				break;

			if (!_tcschr(rootPtr, ';'))
				onLastPath = TRUE;
			}
		if (newNode)
			newNode->iNodeType = 0x4; // last sibling
		}
	else
		{
		iDirTreeRoot->iFileName = _tcsdup(aRoot);
		ReadDir(iDirTreeRoot->iFileName, iDirTreeRoot, aRecursive);
		}
	}

void CBemusedServerDlg::SetSeek(SOCKET aSocket)
	{
	unsigned long bytesRead = 0;
	unsigned char indexBytes[4];
	int seek = 0;

	if (!ReadFromPhone(aSocket, indexBytes, 4))
		{
		Log("Couldn't read seek: %d\r\n", GetLastError());
		return;
		}

	seek = (indexBytes[0] << 24) + (indexBytes[1] << 16) +
		   (indexBytes[2] << 8) + indexBytes[3];

	// set the seek
	iAppHandler->SetSeek(seek);

	Log("Set seek to %d seconds\r\n", seek);
	}

void CBemusedServerDlg::WritePlaylistLength(SOCKET aSocket)
	{
	unsigned long bytesWritten = 0;
	int playlistLength;

	iAppHandler->GetPlaylistLength(playlistLength);

	unsigned char buffer[9];
	strcpy((char*) buffer, "PLENACK");
	buffer[7] = (playlistLength >> 8) & 0xFF;
	buffer[8] = playlistLength & 0xFF;

	if (aSocket)
		bytesWritten = send(aSocket, (char*)buffer, 9, 0);
	else
		WriteFile(iBluetoothHandle, buffer, 9, &bytesWritten, NULL);

	Log("Returned playlist length: %d\r\n", playlistLength);
	}

void CBemusedServerDlg::BluetoothConnect(TCHAR* aCommPort)
	{
	if (iBluetoothHandle != INVALID_HANDLE_VALUE)
		CloseHandle(iBluetoothHandle);

	TCHAR commPortBuf[256];

	if (_tcslen(aCommPort) >= (sizeof(commPortBuf) - 4))
		{
		Log("Comm port length (%d) too long", lstrlen(aCommPort));
		return;
		}

	if (lstrlen(aCommPort) == 0) {
		Log("Comm port needed");
		return;
	}
	if (aCommPort[lstrlen(aCommPort)-1] != ':') {
		_tcscat(aCommPort, _T(":"));
	}

	swprintf(commPortBuf, _T("%s"), aCommPort);

	iBluetoothHandle = CreateFile(commPortBuf,
		GENERIC_READ | GENERIC_WRITE,
		FILE_SHARE_READ | FILE_SHARE_WRITE, // non-exclusive access
		NULL, // no security
		OPEN_EXISTING,
		0, // not overlapped I/O
		NULL);

	if (iBluetoothHandle == INVALID_HANDLE_VALUE)
		{
		if (GetLastError() == 2)
			{
			Log("Couldn't find port \"%S\".\r\n"
				"Did you enter a valid COM port in the format \"COMn\" (where n is a number)?\r\n",
				aCommPort);
			}
		else if (GetLastError() == 5)
			{
			Log("Couldn't open port %S: it appears to be in use. Is PC Suite using this port?\r\n",
				aCommPort);
			}
		else
			{
			Log("Couldn't open port %S (error %d).\r\n"
				"Choose a different port and press \"Connect\".\r\n", aCommPort, GetLastError());
			}
		}
	else
		{
		COMMTIMEOUTS timeouts;
		memset(&timeouts, 0x00, sizeof(timeouts));
		BOOL ret = SetCommTimeouts(iBluetoothHandle, &timeouts);

		if (ret)
			Log("Connected to port %S\r\n", aCommPort);
		else
			Log("Couldn't set timeouts on port %S (error %d).\r\n", aCommPort, GetLastError());
		}
	}

int CBemusedServerDlg::FindCommPortInRegistry()
	{
	HKEY key;

	// See if the Widcomm stack is installed.

	if (RegOpenKeyEx(HKEY_CURRENT_USER, _T("Software\\Widcomm\\BTConfig\\Services"),
		0, KEY_READ, &key) != ERROR_SUCCESS)
		{
		Log("Couldn't automatically find which COM port to use\r\n");
		return FALSE;
		}

	RegCloseKey(key);

	TCHAR keyNameBuf[256];
	DWORD keyNameSize;
	DWORD commPortNum = -1;
	DWORD valueType;

	for (int i = 1; i < 20; i++)
		{
		swprintf(keyNameBuf, _T("Software\\Widcomm\\BTConfig\\Services\\%04d"), i);

		if (RegOpenKeyEx(HKEY_CURRENT_USER, keyNameBuf, 0, KEY_READ, &key) != ERROR_SUCCESS)
			continue;

		keyNameSize = sizeof(keyNameBuf);
		if (RegQueryValueEx(key, _T("Name"), 0, &valueType, (BYTE*)keyNameBuf, &keyNameSize)
			!= ERROR_SUCCESS)
			{
			RegCloseKey(key);
			continue;
			}

		// Perform a crude search for Bemused or virtual serial port services.
		if (_tcsstr(keyNameBuf, _T("emused")) || _tcsstr(keyNameBuf, _T("erial")))
			{
			keyNameSize = sizeof(commPortNum);

			if (RegQueryValueEx(key, _T("ComPortNumber"), 0, &valueType,
				(BYTE*)&commPortNum, &keyNameSize) == ERROR_SUCCESS)
				{
				Log("COM%d might be the port that Bemused should use\r\n", commPortNum);
				}
			}

		RegCloseKey(key);
		}

	// Choose the last COM port we found, if we found any.

	if (commPortNum != -1)
		{
		swprintf(iCommPortName, _T("COM%d"), commPortNum);
		SendDlgItemMessage(IDC_COMPORT, WM_SETTEXT, 0, (LPARAM)iCommPortName);
		return commPortNum;
		}

	Log("Couldn't automatically find which COM port to use\r\n");

	return 0;
	}

CAppHandler* CBemusedServerDlg::FindHandlerForFile(const TCHAR* aFileName)
	{
	// Get the latest info from the combo box.
	OnSelchangeAppcombo();

	TCHAR* dotPos = _tcsrchr(aFileName, '.');
	TCHAR szExt[16];

	iCurrentAppIndex = iDefaultAppIndex;

	if (!dotPos || lstrlen(dotPos) > 15)
		return iAppHandlers[iDefaultAppIndex];

	_tcscpy(szExt, dotPos);
	dotPos = &szExt[0];

	BOOL handlerWantsExtension[NUM_HANDLERS];
	int i;

	for (i = 0; i < lstrlen(szExt); i++) {
		szExt[i] = towlower(szExt[i]);
	}

	for (i = 0; i < NUM_HANDLERS; i++)
		{
		if (_tcsstr(iExtensionsHandled[i], dotPos + 1))
			handlerWantsExtension[i] = TRUE;
		else
			handlerWantsExtension[i] = FALSE;
		}

	// If we have multiple handlers for the same file type, we
	// give preference to the default handler. Otherwise, we use
	// the first handler we find which wants that file type. Finally,
	// we fall back on the default handler.

	if (handlerWantsExtension[iDefaultAppIndex])
		{
		iCurrentAppIndex = iDefaultAppIndex;
		return iAppHandlers[iDefaultAppIndex];
		}

	for (i = 0; i < NUM_HANDLERS; i++)
		{
		if (handlerWantsExtension[i])
			{
			iCurrentAppIndex = i;
			return iAppHandlers[i];
			}
		}

	iCurrentAppIndex = iDefaultAppIndex;
	return iAppHandlers[iDefaultAppIndex];
	}

void CBemusedServerDlg::OnSysCommand(UINT nID, LPARAM lParam)
	{
	if ((nID & 0xFFF0) == IDM_ABOUTBOX)
		{
		CAboutDlg dlgAbout;
		dlgAbout.DoModal();
		}
	else
		{
		CDialog::OnSysCommand(nID, lParam);
		}
	}

// If you add a minimize button to your dialog, you will need the code below
//  to draw the icon.  For MFC applications using the document/view model,
//  this is automatically done for you by the framework.

void CBemusedServerDlg::OnPaint() 
	{
	if (iHidden)
		ShowWindow(SW_HIDE);

	if (IsIconic())
		{
		CPaintDC dc(this); // device context for painting

#ifndef UNDER_CE
		SendMessage(WM_ICONERASEBKGND, (WPARAM) dc.GetSafeHdc(), 0);
#endif

		// Center icon in client rectangle
		int cxIcon = GetSystemMetrics(SM_CXICON);
		int cyIcon = GetSystemMetrics(SM_CYICON);
		CRect rect;
		GetClientRect(&rect);
		int x = (rect.Width() - cxIcon + 1) / 2;
		int y = (rect.Height() - cyIcon + 1) / 2;

		// Draw the icon
		dc.DrawIcon(x, y, m_hIcon);
		}
	else
		{
		CDialog::OnPaint();
		}
	}

// The system calls this to obtain the cursor to display while the user drags
//  the minimized window.
HCURSOR CBemusedServerDlg::OnQueryDragIcon()
	{
	return (HCURSOR) m_hIcon;
	}

void CBemusedServerDlg::OnReconnect() 
	{
	SendDlgItemMessage(IDC_COMPORT, WM_GETTEXT, sizeof(iCommPortName)/sizeof(TCHAR), (LPARAM)iCommPortName);

	TCHAR* commPortPtr = iCommPortName;
	int commPortNum = -1;

	while (*commPortPtr != '\0')
		{
		commPortNum = _ttoi(commPortPtr);
		if (commPortNum > 0)
			{
			swprintf(iCommPortName, _T("COM%d"), commPortNum);
			break;
			}
		commPortPtr++;
		}

	if (iBluetoothThread)
		{
		TerminateThread(iBluetoothThread->m_hThread, 0);
		delete iBluetoothThread;
		iBluetoothThread = NULL;
		}

	if (iBluetoothHandle != INVALID_HANDLE_VALUE)
		{
		CloseHandle(iBluetoothHandle);
		iBluetoothHandle = INVALID_HANDLE_VALUE;
		}

	BluetoothConnect(iCommPortName);

	if (iBluetoothHandle != INVALID_HANDLE_VALUE)
		iBluetoothThread = AfxBeginThread(BluetoothThreadFunction, this);
	}

void CBemusedServerDlg::Log(char* aFormatString, ...)
	{
	char logBuffer[1024];

	va_list argList;
	va_start(argList, aFormatString);
	vsprintf(logBuffer, aFormatString, argList);
	va_end(argList);

	int length = SendDlgItemMessage(IDC_LOGBOX, WM_GETTEXTLENGTH, 0, 0);
	SendDlgItemMessage(IDC_LOGBOX, EM_SETSEL, length, length);
	SendDlgItemMessage(IDC_LOGBOX, EM_REPLACESEL, 0, (LPARAM) (LPCTSTR)CString(logBuffer));
#ifdef DEBUG
	OutputDebugString(CString(logBuffer) + CString(_T("\r\n")));
#endif
	}

void CBemusedServerDlg::OnClose() 
	{
	Shell_NotifyIcon(NIM_DELETE, &iNotifyIconData);
	WriteIniFile();
	CDialog::OnClose();
	}

void CBemusedServerDlg::OnExit() 
	{
	Shell_NotifyIcon(NIM_DELETE, &iNotifyIconData);
	WriteIniFile();
	EndDialog(0);
	}

void CBemusedServerDlg::OnSize(UINT nType, int cx, int cy) 
	{
	CDialog::OnSize(nType, cx, cy);
	
	if (nType == SIZE_MINIMIZED)
		{
		ShowWindow(SW_HIDE);
		iHidden = TRUE;
		}
	}

void CBemusedServerDlg::OnOK()
	{
		ShowWindow(SW_HIDE);
		iHidden = TRUE;
	}

void CBemusedServerDlg::OnBrowsePath() 
	{
#ifndef UNDER_CE
	TCHAR rootPath[MAX_PATH];

	LPITEMIDLIST itemList;
	BROWSEINFO browseInfo;
	browseInfo.hwndOwner = m_hWnd;
	browseInfo.pidlRoot = NULL; 
    browseInfo.pszDisplayName = rootPath; 
    browseInfo.lpszTitle = "Select the folder which contains your music.";
    browseInfo.ulFlags = 0;
    browseInfo.lpfn = NULL;
    browseInfo.lParam = 0;
    browseInfo.iImage = 0;
	
	// Display the folder browser.
	itemList = SHBrowseForFolder(&browseInfo);

	if (itemList)
		{
		SHGetPathFromIDList(itemList, iRootPath);
		SendDlgItemMessage(IDC_EDITPATH, WM_SETTEXT, 0, (LPARAM)iRootPath);

		// Free the ID list structure.
		LPMALLOC mallocPtr;
		SHGetMalloc(&mallocPtr);
		mallocPtr->Free(itemList);
		}
#endif
	}

void CBemusedServerDlg::OnBrowseWinamp() 
	{
	OPENFILENAME ofn;       // common dialog box structure
	TCHAR path[260];       // buffer for filename

	SendDlgItemMessage(IDC_WINAMPPATH, WM_GETTEXT, sizeof(path)/sizeof(TCHAR), (LPARAM) path);

	// Initialise structure.
	ZeroMemory(&ofn, sizeof(OPENFILENAME));
	ofn.lStructSize = sizeof(OPENFILENAME);
	ofn.hwndOwner = m_hWnd;
	ofn.lpstrFile = path;
	ofn.nMaxFile = sizeof(path);
	ofn.lpstrFilter = NULL;
	ofn.nFilterIndex = 0;
	ofn.lpstrFileTitle = NULL;
	ofn.nMaxFileTitle = 0;
	ofn.lpstrInitialDir = NULL;
	ofn.Flags = OFN_HIDEREADONLY |    // Hide read-only prompt
                OFN_OVERWRITEPROMPT | // Hide Over-write prompt
                OFN_LONGNAMES  |      // Allow long file names
                OFN_NOVALIDATE |      // Don't validate
                OFN_PATHMUSTEXIST;

	// Display the Open dialog box and get a filename.
	if (GetOpenFileName(&ofn))
		SendDlgItemMessage(IDC_WINAMPPATH, WM_SETTEXT, 0, (LPARAM)path);
	}

void CBemusedServerDlg::OnAddpath() 
	{
#ifndef UNDER_CE
	TCHAR rootPath[MAX_PATH];
	LPITEMIDLIST itemList;
	BROWSEINFO browseInfo;
	browseInfo.hwndOwner = m_hWnd;
	browseInfo.pidlRoot = NULL; 
    browseInfo.pszDisplayName = rootPath; 
    browseInfo.lpszTitle = "Select a folder to add to the list.";
    browseInfo.ulFlags = 0;
    browseInfo.lpfn = NULL;
    browseInfo.lParam = 0;
    browseInfo.iImage = 0;
	
	// Display the folder browser.
	itemList = SHBrowseForFolder(&browseInfo);

	if (itemList)
		{
		SHGetPathFromIDList(itemList, rootPath);

		if (_tcscmp(iRootPath, _T("")))
			_tcscat(iRootPath, _T(";"));
		_tcscat(iRootPath, rootPath);
		SendDlgItemMessage(IDC_EDITPATH, WM_SETTEXT, 0, (LPARAM)iRootPath);

		// Free the ID list structure.
		LPMALLOC mallocPtr;
		SHGetMalloc(&mallocPtr);
		mallocPtr->Free(itemList);
		}
#endif
	}

void CBemusedServerDlg::OnClearpath() 
	{
	_tcscpy(iRootPath, _T(""));
	SendDlgItemMessage(IDC_EDITPATH, WM_SETTEXT, 0, (LPARAM)iRootPath);
	}

void CBemusedServerDlg::OnSelchangeAppcombo() 
	{
	SendDlgItemMessage(IDC_FILETYPESEDIT, WM_GETTEXT,
		sizeof(iExtensionsHandled[iDefaultAppIndex])/sizeof(TCHAR), (LPARAM) iExtensionsHandled[iDefaultAppIndex]);

	SendDlgItemMessage(IDC_WINAMPPATH, WM_GETTEXT,
		sizeof(iAppPaths[iDefaultAppIndex])/sizeof(TCHAR), (LPARAM) iAppPaths[iDefaultAppIndex]);

	iDefaultAppIndex = SendDlgItemMessage(IDC_APPCOMBO, CB_GETCURSEL, 0, 0);

	CAppHandler* handler = iAppHandlers[iDefaultAppIndex];

	if (handler != iAppHandler)
		{
		int stopWhenSwitching = SendDlgItemMessage(IDC_STOPWHENSWITCHING, BM_GETCHECK, 0, 0);

		if (stopWhenSwitching)
			Stop(EImmediate);
		}

	iAppHandler = handler;

	SendDlgItemMessage(IDC_FILETYPESEDIT, WM_SETTEXT, 0,
		(LPARAM) iExtensionsHandled[iDefaultAppIndex]);

	SendDlgItemMessage(IDC_WINAMPPATH, WM_SETTEXT, 0,
		(LPARAM) iAppPaths[iDefaultAppIndex]);
	}

void CBemusedServerDlg::OnFindcomport() 
	{
	FindCommPortInRegistry();
	}

void CBemusedServerDlg::OnActivate(UINT nState, CWnd *pWndOther, BOOL bMinimized)
	{
		if (nState == WA_ACTIVE || nState == WA_CLICKACTIVE) {
			if (iHidden) {
				PostMessage(WM_ICONNOTIFY, 0, WM_LBUTTONUP);
			}
		}
	}

void CBemusedServerDlg::OnEndSession(BOOL bEnding) 
	{
	WriteIniFile();
#ifndef UNDER_CE
	CDialog::OnEndSession(bEnding);
#endif
	}

void CBemusedServerDlg::InitialiseSettingsFromWizard(int aDefaultApplication, const TCHAR* aAppPath,
		const TCHAR* aMusicPath, const TCHAR* aCommPortName)
	{
	TCHAR path[MAX_PATH + 20];

	_tcscpy(path, iStartDirectory);
	_tcscat(path, KIniFileName);

	TCHAR buffer[256];

	WritePrivateProfileString(KIniFileSection, _T("CommPort"), aCommPortName, path);
	WritePrivateProfileString(KIniFileSection, _T("Path"), aMusicPath, path);
	WritePrivateProfileString(KIniFileSection, _T("StartWinampAutomatically"), _T("1"), path);
	swprintf(buffer, _T("%d"), aDefaultApplication);
	WritePrivateProfileString(KIniFileSection, _T("DefaultApplication"), buffer, path);

	for (int i = 0; i < NUM_HANDLERS; i++)
		{
		swprintf(buffer, _T("PathToApplication%d"), i);

		if (i == 1)
			{
			WritePrivateProfileString(KIniFileSection, buffer, aAppPath, path);
			swprintf(buffer, _T("ExtensionsHandled%d"), i);
			WritePrivateProfileString(KIniFileSection, buffer, _T("mp3;mp2;wav;wma;ogg"), path);
			}
		else
			{
			WritePrivateProfileString(KIniFileSection, buffer, _T(""), path);
			if (i == 0) // PowerPoint
				{
				WritePrivateProfileString(KIniFileSection, _T("ExtensionsHandled3"), _T("cpt"), path);
				}
			else
				{
				swprintf(buffer, _T("ExtensionsHandled%d"), i);
				WritePrivateProfileString(KIniFileSection, buffer, _T(""), path);
				}
			}
		}

	WritePrivateProfileString(KIniFileSection, _T("StartMinimised"), _T("0"), path);

	// Read everything back in for a sensible startup.
	ReadIniFile();
	}

void CBemusedServerDlg::OnStopwhenswitching() 
	{
	}

void CBemusedServerDlg::OnListenonsocket() 
	{
	iListenOnSocket = SendDlgItemMessage(IDC_LISTENONSOCKET, BM_GETCHECK, 0, 0);

	if (iListenOnSocket)
		ListenOnSocket();
	else
		StopListeningOnSocket();
	}
